==> gist-angular-popovers-1.0.5.tgz: package/src/ts/gist-popovers/directives/gist-popover.directive.ts <==
import { GistPopoverService } from './../services/gist-popover.service';
import { PopoverEntry } from './../classes/popover-entry.class';
import { Directive, Input, TemplateRef, ViewContainerRef, ElementRef } from '@angular/core';
import { Subscription } from 'rxjs/Subscription';
import { Observable } from 'rxjs/Observable';

let offset = 15;
interface IValidInput {
    key: string;
    default: any;
    parse( val: any ): any;
}
let validInputs: IValidInput[] = [
    {
        key: 'target',
        default: 'ele',
        parse: ( val: any ) => {
            let opts = [ 'ele', 'mouse' ]; // or x,y where x and y are numbers
            let isValid = false;
            if ( opts.indexOf( val ) !== -1 ) {
                isValid = true;
            }
            else if ( val.indexOf( ',' ) ) {
                let coords = val.split( ',' );
                if ( coords.length === 2 ) {
                    let isXnan = isNaN( parseFloat( coords[ 0 ] ) );
                    let isYnan = isNaN( parseFloat( coords[ 1 ] ) );
                    isValid = !isXnan && !isYnan;
                }
            }
            if ( !isValid ) {
                console.error( 'gistPopover invalid paramater value. Target must be one of ' + opts.join( ', ' ) + ', XX,YY ( as page coordinates). Value given: ' + val );
                return;
            }
            return val;
        },
    },
    {
        key: 'position',
        default: 'bottom',
        parse: ( val: any ) => {
            let opts = [ 'top', 'bottom', 'left', 'right' ];
            if ( opts.indexOf( val ) === -1 ) {
                console.error( 'gistPopover invalid paramater value. Position must be one of ' + opts.join( ', ' ) + '. Value given: ' + val );
                return;
            }
            return val;
        }
    },
    {
        key: 'isInteractive',
        default: false,
        parse: ( val: any ) => {
            if ( val === 'true' ) {
                return true;
            } else if ( val === 'false' ) {
                return false;
            } else {
                console.error( 'gistPopover invalid paramater value. Position must be the string "true" or "false". Value given: ' + val );
            }
        }
    },
    {
        key: 'isVisible',
        default: undefined,
        parse: ( val: any ) => {
            if ( val === 'true' ) {
                return true;
            } else if ( val === 'false' ) {
                return false;
            }
            else if ( val === 'undefined' ) {
                return undefined;
            } else {
                console.error( 'gistPopover invalid paramater value. Position must be the string "true", "false", or "undefined". Value given: ' + val );
                return;
            }
        }
    },
    {
        key: 'delay',
        default: 100,
        parse: ( val: any ) => {
            let tmp = +val;
            if ( tmp.toString() !== val.toString() || tmp < 0 ) {
                console.error( 'gistPopover invalid paramater value. Delay must be a number >= 0' );
                return;
            }
        }
    }
];

@Directive( { selector: '[gistPopover]' } )
export class GistPopoverDirective {
    private windowSubs: Subscription[] = [];
    private mouseSubs: Subscription[] = [];
    private isReady: boolean = false;
    private entry: PopoverEntry;
    private ele: HTMLElement;

    private target: string; // 'ele' | 'mouse';
    private position: string; // 'atuo'| 'top' | 'bottom' | 'left' | 'right';
    private delay: number; // any positive number
    private isInteractive: boolean; // truthy value
    private isVisible: boolean; // truthy value or undefined

    private hideTimer: number = undefined;
    private showTimer: number = undefined;

    constructor(
        private templateRef: TemplateRef<any>,
        private viewContainer: ViewContainerRef,
        private popService: GistPopoverService,
        private eleRef: ElementRef
    ) { }

    ngAfterViewInit() {
        let me = this;
        me.ele = me.eleRef.nativeElement;

        // take the content out of the view
        me.viewContainer.clear();

        me.entry = me.popService.registerPopover( me.templateRef );

        me.windowSubs.push(
            Observable.fromEvent( window.document, 'resize' ).subscribe(( e: any ) => me.updatePopover.apply( me ) )
        );
        me.isReady = true;
    }

    @Input()
    set gistPopover( inputString: string ) {
        let me = this;
        let inputs = inputString ? inputString.split( ';' ) : [];
        validInputs.forEach( vi => { me[ vi.key ] = undefined; } );

        inputs.forEach(
            ( input: string ) => {
                // reset inputs
                let pv = input.split( ':' );
                if ( pv.length !== 2 ) {
                    return;
                }
                let prop = pv[ 0 ].trim();
                let val = pv[ 1 ].trim();

                let myProp = validInputs.filter(( vi: { key: string } ) => { return vi.key === prop; } )[ 0 ];
                if ( !myProp ) {
                    throw 'gistPopover invalid property: ' + prop + '. ' +
                    'Valid properties are ' + validInputs.map(( vi: { key: string } ) => { return vi.key; } ).join( ', ' );
                }
                try {
                    me[ prop ] = myProp.parse( val );
                } catch ( e ) {
                    console.error( e );
                    me[ prop ] = undefined;
                }

            }
        );
        validInputs.forEach(( vi: { key: string, default: any } ) => {
            if ( me[ vi.key ] === undefined ) {
                me[ vi.key ] = vi.default;
            }
        } );

        me.waitTillReady( me.processInput );
    }

    private processInput() {
        let me = this;
        // track mouse if isVisisble is not defiend
        if ( me.target === 'mouse' || me.isVisible === undefined ) {
            // start listening if were not
            if ( me.mouseSubs.length === 0 ) {
                me.trackMouse();
            }
        } else {
            if ( me.mouseSubs.length !== 0 ) {
                me.unTrackMouse();
            }
        }
        if ( me.isVisible !== undefined ) {
            if ( me.isVisible ) {
                me.showPopover();
            } else {
                me.hidePopover();
            }
        }


        if ( me.target.indexOf( ',' ) !== -1 ) {
            me.updatePopover();
        }
    }


    private trackMouse() {
        let me = this;
        let pe = me.ele.parentElement;
        if ( pe ) {
            me.mouseSubs.push(
                Observable.fromEvent( pe, 'mousemove' ).subscribe(( e: MouseEvent ) => {
                    if ( me.isVisible || me.isVisible === undefined ) {
                        me.showPopover.apply( me, [ e ] );
                    }
                } )
            );
            me.mouseSubs.push(
                Observable.fromEvent( pe, 'mouseleave' ).subscribe(( e: MouseEvent ) => {
                    if ( !me.isVisible ) {
                        me.hidePopover.apply( me );
                    }
                } )
            );
        } else {
            console.warn( 'no element to listen to.' );
        }

    }

    private unTrackMouse() {
        this.mouseSubs.forEach( s => s.unsubscribe() );
    }

    private updatePopover( e?: MouseEvent ) {
        if ( this.entry && this.entry.isShown ) {
            this.showPopover( e );
        }
    }

    private showPopover( e?: MouseEvent ) {
        let me = this;
        function display( coords: number[] ) {
            me.entry.isShown = true;
            me.entry.xPos.next( coords[ 0 ] );
            me.entry.yPos.next( coords[ 1 ] );
        }

        me.entry.delay = me.delay;
        me.entry.isInteractive = me.isInteractive;
        me.entry.anchorSide.next( me.position );

        let coords: number[];
        if ( me.target === 'mouse' ) {
            if ( e ) {
                coords = me.getMouseAnchor( e );
            }
        }
        else if ( me.target === 'ele' ) {
            coords = me.getEleAnchor( me.ele.parentElement );

        } else {
            coords = me.getTargetAnchor();
        }

        if ( this.entry.isShown && coords ) {
            display( coords );
        }

        if ( this.hideTimer ) {
            window.clearTimeout( this.hideTimer );
            this.hideTimer = undefined;
        }
        if ( !me.entry.isShown && !this.showTimer && coords ) {
            this.showTimer = window.setTimeout(() => {
                display( coords );
                this.showTimer = undefined;
            }, this.entry.delay );
        }
    }

    getMouseAnchor( event: MouseEvent ): number[] {
        let xPos: number = event.pageX;
        let yPos: number = event.pageY;

        let xOffset: number = offset;
        let yOffset: number = offset;


        if ( this.position === 'left' ) {
            xOffset = offset * -1;
        }
        if ( this.position === 'top' ) {
            yOffset = offset * -1;
        }

        return [ xPos + xOffset, yPos + yOffset ];
    }

    getEleAnchor( ele: HTMLElement ): number[] {
        let bounds = ele.getBoundingClientRect();
        let xPos: number = bounds.left + bounds.width / 2;
        let yPos: number = bounds.top + bounds.height / 2;

        if ( this.position === 'left' ) {
            xPos = bounds.left - offset;
        }
        else if ( this.position === 'right' ) {
            xPos = bounds.right + offset;
        }
        if ( this.position === 'top' ) {
            yPos = bounds.top - offset;
        }
        else if ( this.position === 'bottom' ) {
            yPos = bounds.bottom + offset;
        }
        return [ xPos, yPos ];
    }

    getTargetAnchor(): number[] {
        return this.target.split( ',' ).map( x => +x );
    }

    private hidePopover() {
        let me = this;
        if ( this.showTimer !== undefined ) {
            clearTimeout( this.showTimer );
            this.showTimer = undefined;
        }
        if ( me.entry.isShown && !this.hideTimer ) {
            this.hideTimer = window.setTimeout(() => {
                this.entry.isShown = false;
                this.hideTimer = undefined;
            }, this.entry.delay );
        }
    }

    ngOnDestroy() {
        this.popService.unRegisterPopover( this.entry );
        this.mouseSubs.forEach( s => s.unsubscribe() );
        this.windowSubs.forEach( s => s.unsubscribe() );
    }

    private waitTillReady( cb: () => any ) {
        let me = this;
        if ( me.isReady ) {
            cb.apply( me );
        } else {
            window.setTimeout(() => {
                me.waitTillReady.apply( me, [ cb ] );
            }, 1 );
        }
    }
}

==> gist-angular-popovers-1.0.10.tgz: package/src/ts/gist-popovers/directives/gist-popover.directive.ts <==
import { GistPopoverService } from './../services/gist-popover.service';
import { PopoverEntry } from './../classes/popover-entry.class';
import { Directive, Input, TemplateRef, ViewContainerRef, ElementRef } from '@angular/core';
import { Subscription } from 'rxjs/Subscription';
import { Observable } from 'rxjs/Observable';

let offset = 15;
interface IValidInput {
    key: string;
    default: any;
    parse( val: any ): any;
}
let validInputs: IValidInput[] = [
    {
        key: 'target',
        default: 'ele',
        parse: ( val: any ) => {
            let opts = [ 'ele', 'mouse' ]; // or x,y where x and y are numbers
            let isValid = false;
            if ( opts.indexOf( val ) !== -1 ) {
                isValid = true;
            }
            else if ( val.indexOf( ',' ) ) {
                let coords = val.split( ',' );
                if ( coords.length === 2 ) {
                    let isXnan = isNaN( parseFloat( coords[ 0 ] ) );
                    let isYnan = isNaN( parseFloat( coords[ 1 ] ) );
                    isValid = !isXnan && !isYnan;
                }
            }
            if ( !isValid ) {
                console.error( 'gistPopover invalid paramater value. Target must be one of ' + opts.join( ', ' ) + ', XX,YY ( as page coordinates). Value given: ' + val );
                return;
            }
            return val;
        },
    },
    {
        key: 'position',
        default: 'bottom',
        parse: ( val: any ) => {
            let opts = [ 'top', 'bottom', 'left', 'right' ];
            if ( opts.indexOf( val ) === -1 ) {
                console.error( 'gistPopover invalid paramater value. Position must be one of ' + opts.join( ', ' ) + '. Value given: ' + val );
                return;
            }
            return val;
        }
    },
    {
        key: 'isInteractive',
        default: false,
        parse: ( val: any ) => {
            if ( val === 'true' ) {
                return true;
            } else if ( val === 'false' ) {
                return false;
            } else {
                console.error( 'gistPopover invalid paramater value. Position must be the string "true" or "false". Value given: ' + val );
            }
        }
    },
    {
        key: 'isVisible',
        default: undefined,
        parse: ( val: any ) => {
            if ( val === 'true' ) {
                return true;
            } else if ( val === 'false' ) {
                return false;
            }
            else if ( val === 'undefined' ) {
                return undefined;
            } else {
                console.error( 'gistPopover invalid paramater value. Position must be the string "true", "false", or "undefined". Value given: ' + val );
                return;
            }
        }
    },
    {
        key: 'delay',
        default: 100,
        parse: ( val: any ) => {
            let tmp = +val;
            if ( tmp.toString() !== val.toString() || tmp < 0 ) {
                console.error( 'gistPopover invalid paramater value. Delay must be a number >= 0' );
                return;
            }
        }
    }
];

@Directive( { selector: '[gistPopover]' } )
export class GistPopoverDirective {
    private windowSubs: Subscription[] = [];
    private mouseSubs: Subscription[] = [];
    private isReady: boolean = false;
    private entry: PopoverEntry;
    private ele: HTMLElement;

    private target: string; // 'ele' | 'mouse';
    private position: string; // 'atuo'| 'top' | 'bottom' | 'left' | 'right';
    private delay: number; // any positive number
    private isInteractive: boolean; // truthy value
    private isVisible: boolean; // truthy value or undefined

    private hideTimer: number = undefined;
    private showTimer: number = undefined;

    constructor(
        private templateRef: TemplateRef<any>,
        private viewContainer: ViewContainerRef,
        private popService: GistPopoverService,
        private eleRef: ElementRef
    ) { }

    ngAfterViewInit() {
        let me = this;
        me.ele = me.eleRef.nativeElement;

        // take the content out of the view
        me.viewContainer.clear();

        me.entry = me.popService.registerPopover( me.templateRef );

        me.windowSubs.push(
            Observable.fromEvent( window.document, 'resize' ).subscribe(( e: any ) => me.updatePopover.apply( me ) )
        );
        me.isReady = true;
    }

    @Input()
    set gistPopover( inputString: string ) {
        let me = this;
        let inputs = inputString ? inputString.split( ';' ) : [];
        validInputs.forEach( vi => { me[ vi.key ] = undefined; } );

        inputs.forEach(
            ( input: string ) => {
                // reset inputs
                let pv = input.split( ':' );
                if ( pv.length !== 2 ) {
                    return;
                }
                let prop = pv[ 0 ].trim();
                let val = pv[ 1 ].trim();

                let myProp = validInputs.filter(( vi: { key: string } ) => { return vi.key === prop; } )[ 0 ];
                if ( !myProp ) {
                    throw 'gistPopover invalid property: ' + prop + '. ' +
                    'Valid properties are ' + validInputs.map(( vi: { key: string } ) => { return vi.key; } ).join( ', ' );
                }
                try {
                    me[ prop ] = myProp.parse( val );
                } catch ( e ) {
                    console.error( e );
                    me[ prop ] = undefined;
                }

            }
        );
        validInputs.forEach(( vi: { key: string, default: any } ) => {
            if ( me[ vi.key ] === undefined ) {
                me[ vi.key ] = vi.default;
            }
        } );

        me.waitTillReady( me.processInput );
    }

    private processInput() {
        let me = this;
        // track mouse if isVisisble is not defiend
        if ( me.target === 'mouse' || me.isVisible === undefined ) {
            // start listening if were not
            if ( me.mouseSubs.length === 0 ) {
                me.trackMouse();
            }
        } else {
            if ( me.mouseSubs.length !== 0 ) {
                me.unTrackMouse();
            }
        }
        if ( me.isVisible !== undefined ) {
            if ( me.isVisible ) {
                me.showPopover();
            } else {
                me.hidePopover();
            }
        }


        if ( me.isVisible && me.target.indexOf( ',' ) !== -1 ) {
            me.updatePopover();
        }
    }


    private trackMouse() {
        let me = this;
        let pe = me.ele.parentElement;
        if ( pe ) {
            me.mouseSubs.push(
                Observable.fromEvent( pe, 'mousemove' ).subscribe(( e: MouseEvent ) => {
                    if ( me.isVisible || me.isVisible === undefined ) {
                        me.showPopover.apply( me, [ e ] );
                    }
                } )
            );
            me.mouseSubs.push(
                Observable.fromEvent( pe, 'mouseleave' ).subscribe(( e: MouseEvent ) => {
                    if ( !me.isVisible ) {
                        me.hidePopover.apply( me );
                    }
                } )
            );
        } else {
            console.warn( 'no element to listen to.' );
        }

    }

    private unTrackMouse() {
        this.mouseSubs.forEach( s => s.unsubscribe() );
    }

    private updatePopover( e?: MouseEvent ) {
        if ( this.entry && this.entry.isShown ) {
            this.showPopover( e );
        }
    }

    private showPopover( e?: MouseEvent ) {
        let me = this;
        function display( coords: number[] ) {
            me.entry.isShown = true;
            me.entry.xPos.next( coords[ 0 ] );
            me.entry.yPos.next( coords[ 1 ] );
        }

        me.entry.delay = me.delay;
        me.entry.isInteractive = me.isInteractive;
        me.entry.anchorSide.next( me.position );

        let coords: number[];
        if ( me.target === 'mouse' ) {
            if ( e ) {
                coords = me.getMouseAnchor( e );
            }
        }
        else if ( me.target === 'ele' ) {
            coords = me.getEleAnchor( me.ele.parentElement );

        } else {
            coords = me.getTargetAnchor();
        }

        if ( this.entry.isShown && coords ) {
            display( coords );
        }

        if ( this.hideTimer ) {
            window.clearTimeout( this.hideTimer );
            this.hideTimer = undefined;
        }
        if ( !me.entry.isShown && !this.showTimer && coords ) {
            this.showTimer = window.setTimeout(() => {
                display( coords );
                this.showTimer = undefined;
            }, this.entry.delay );
        }
    }

    getMouseAnchor( event: MouseEvent ): number[] {
        let xPos: number = event.pageX;
        let yPos: number = event.pageY;

        let xOffset: number = offset;
        let yOffset: number = offset;


        if ( this.position === 'left' ) {
            xOffset = offset * -1;
        }
        if ( this.position === 'top' ) {
            yOffset = offset * -1;
        }

        return [ xPos + xOffset, yPos + yOffset ];
    }

    getEleAnchor( ele: HTMLElement ): number[] {
        let bounds = ele.getBoundingClientRect();
        let xPos: number = bounds.left + bounds.width / 2;
        let yPos: number = bounds.top + bounds.height / 2;

        if ( this.position === 'left' ) {
            xPos = bounds.left - offset;
        }
        else if ( this.position === 'right' ) {
            xPos = bounds.right + offset;
        }
        if ( this.position === 'top' ) {
            yPos = bounds.top - offset;
        }
        else if ( this.position === 'bottom' ) {
            yPos = bounds.bottom + offset;
        }
        return [ xPos, yPos ];
    }

    getTargetAnchor(): number[] {
        return this.target.split( ',' ).map( x => +x );
    }

    private hidePopover() {
        let me = this;
        if ( this.showTimer !== undefined ) {
            clearTimeout( this.showTimer );
            this.showTimer = undefined;
        }
        if ( me.entry.isShown && !this.hideTimer ) {
            this.hideTimer = window.setTimeout(() => {
                this.entry.isShown = false;
                this.hideTimer = undefined;
            }, this.entry.delay );
        }
    }

    ngOnDestroy() {
        this.popService.unRegisterPopover( this.entry );
        this.mouseSubs.forEach( s => s.unsubscribe() );
        this.windowSubs.forEach( s => s.unsubscribe() );
    }

    private waitTillReady( cb: () => any ) {
        let me = this;
        if ( me.isReady ) {
            cb.apply( me );
        } else {
            window.setTimeout(() => {
                me.waitTillReady.apply( me, [ cb ] );
            }, 1 );
        }
    }
}
